@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import datetime
-import sys
 from collections.abc import Callable, Collection, Hashable, Iterator, Mapping, Sequence
 from types import EllipsisType
 from typing import (
@@ -9,7 +8,9 @@ from typing import (
     Any,
     Literal,
     Protocol,
+    Self,
     SupportsIndex,
+    TypeAlias,
     TypeVar,
     Union,
     overload,
@@ -18,21 +19,6 @@ from typing import (
 
 import numpy as np
 import pandas as pd
-
-try:
-    if sys.version_info >= (3, 11):
-        from typing import Self, TypeAlias
-    else:
-        from typing import TypeAlias
-
-        from typing_extensions import Self
-except ImportError:
-    if TYPE_CHECKING:
-        raise
-    else:
-        Self: Any = None
-
-
 from numpy._typing import _SupportsDType
 from numpy.typing import ArrayLike
 
@@ -46,7 +32,7 @@ if TYPE_CHECKING:
     from xarray.core.indexes import Index, Indexes
     from xarray.core.utils import Frozen
     from xarray.core.variable import IndexVariable, Variable
-    from xarray.groupers import Grouper, TimeResampler
+    from xarray.groupers import Grouper, Resampler
     from xarray.structure.alignment import Aligner
 
     GroupInput: TypeAlias = (
@@ -195,7 +181,7 @@ T_DataWithCoords = TypeVar("T_DataWithCoords", bound="DataWithCoords")
 
 # Temporary placeholder for indicating an array api compliant type.
 # hopefully in the future we can narrow this down more:
-T_DuckArray = TypeVar("T_DuckArray", bound=Any, covariant=True)
+T_DuckArray = TypeVar("T_DuckArray", bound=Any, covariant=True)  # noqa: PLC0105
 
 # For typing pandas extension arrays.
 T_ExtensionArray = TypeVar("T_ExtensionArray", bound=pd.api.extensions.ExtensionArray)
@@ -214,8 +200,8 @@ Dims = Union[str, Collection[Hashable], EllipsisType, None]
 
 # FYI in some cases we don't allow `None`, which this doesn't take account of.
 # FYI the `str` is for a size string, e.g. "16MB", supported by dask.
-T_ChunkDim: TypeAlias = str | int | Literal["auto"] | None | tuple[int, ...]  # noqa: PYI051
-T_ChunkDimFreq: TypeAlias = Union["TimeResampler", T_ChunkDim]
+T_ChunkDim: TypeAlias = str | int | Literal["auto"] | tuple[int, ...] | None  # noqa: PYI051
+T_ChunkDimFreq: TypeAlias = Union["Resampler", T_ChunkDim]
 T_ChunksFreq: TypeAlias = T_ChunkDim | Mapping[Any, T_ChunkDimFreq]
 # We allow the tuple form of this (though arguably we could transition to named dims only)
 T_Chunks: TypeAlias = T_ChunkDim | Mapping[Any, T_ChunkDim]
@@ -253,16 +239,16 @@ InterpolantOptions = Literal[
 InterpnOptions = Literal["linear", "nearest", "slinear", "cubic", "quintic", "pchip"]
 InterpOptions = Union[Interp1dOptions, InterpolantOptions, InterpnOptions]
 
-DatetimeUnitOptions = Literal[
-    "W", "D", "h", "m", "s", "ms", "us", "μs", "ns", "ps", "fs", "as", None
-]
+DatetimeUnitOptions = (
+    Literal["W", "D", "h", "m", "s", "ms", "us", "μs", "ns", "ps", "fs", "as"] | None
+)
 NPDatetimeUnitOptions = Literal["D", "h", "m", "s", "ms", "us", "ns"]
 PDDatetimeUnitOptions = Literal["s", "ms", "us", "ns"]
 
-QueryEngineOptions = Literal["python", "numexpr", None]
+QueryEngineOptions = Literal["python", "numexpr"] | None
 QueryParserOptions = Literal["pandas", "python"]
 
-ReindexMethodOptions = Literal["nearest", "pad", "ffill", "backfill", "bfill", None]
+ReindexMethodOptions = Literal["nearest", "pad", "ffill", "backfill", "bfill"] | None
 
 PadModeOptions = Literal[
     "constant",
@@ -281,7 +267,7 @@ T_VarPadConstantValues = T_PadConstantValues | Mapping[Any, T_PadConstantValues]
 T_DatasetPadConstantValues = (
     T_VarPadConstantValues | Mapping[Any, T_VarPadConstantValues]
 )
-PadReflectOptions = Literal["even", "odd", None]
+PadReflectOptions = Literal["even", "odd"] | None
 
 CFCalendar = Literal[
     "standard",
@@ -299,10 +285,10 @@ CoarsenBoundaryOptions = Literal["exact", "trim", "pad"]
 SideOptions = Literal["left", "right"]
 InclusiveOptions = Literal["both", "neither", "left", "right"]
 
-ScaleOptions = Literal["linear", "symlog", "log", "logit", None]
-HueStyleOptions = Literal["continuous", "discrete", None]
+ScaleOptions = Literal["linear", "symlog", "log", "logit"] | None
+HueStyleOptions = Literal["continuous", "discrete"] | None
 AspectOptions = Union[Literal["auto", "equal"], float, None]
-ExtendOptions = Literal["neither", "both", "min", "max", None]
+ExtendOptions = Literal["neither", "both", "min", "max"] | None
 
 
 _T_co = TypeVar("_T_co", covariant=True)
@@ -318,6 +304,10 @@ class NestedSequence(Protocol[_T_co]):
     def __reversed__(self, /) -> Iterator[_T_co | NestedSequence[_T_co]]: ...
 
 
+_T = TypeVar("_T")
+NestedDict = dict[str, "NestedDict[_T] | _T"]
+
+
 AnyStr_co = TypeVar("AnyStr_co", str, bytes, covariant=True)
 
 
@@ -378,3 +368,14 @@ Bins = Union[
 ]
 
 ResampleCompatible: TypeAlias = str | datetime.timedelta | pd.Timedelta | pd.DateOffset
+
+
+class Closable(Protocol):
+    def close(self) -> None: ...
+
+
+class Lock(Protocol):
+    def acquire(self, *args, **kwargs) -> Any: ...
+    def release(self) -> None: ...
+    def __enter__(self) -> Any: ...
+    def __exit__(self, *args, **kwargs) -> None: ...
